<?php

/**
 * HTTP1.1 Client Cache Features
 *
 * @package redaxo4
 * @version $Id: function_rex_client_cache.inc.php,v 1.6 2007/10/13 13:52:01 kills Exp $
 */

/**
 * Sendet den Content zum Client,
 * fgt ggf. HTTP1.1 cache headers hinzu
 *
 * @param $REX_ARTICLE rex_article Den zu sendenen Artikel
 * @param $content string Inhalt des Artikels
 * @param $environment string Die Umgebung aus der der Inhalt gesendet wird
 * (frontend/backend)
 */
function rex_send_content($REX_ARTICLE, $content, $environment)
{
  global $REX;

  // ----- EXTENSION POINT
  $content = rex_register_extension_point( 'OUTPUT_FILTER', $content);

  // ----- EXTENSION POINT - keine Manipulation der Ausgaben ab hier (read only)
  rex_register_extension_point( 'OUTPUT_FILTER_CACHE', $content, '', true);

  // ----- Last-Modified
  if($REX['USE_LAST_MODIFIED'] === 'true' || $REX['USE_LAST_MODIFIED'] == $environment)
    rex_send_last_modified($REX_ARTICLE);

  // ----- ETAG
  if($REX['USE_ETAG'] === 'true' || $REX['USE_ETAG'] == $environment)
    rex_send_etag($content);

  // ----- GZIP
  if($REX['USE_GZIP'] === 'true' || $REX['USE_GZIP'] == $environment)
    $content = rex_send_gzip($content);

  // Evtl offene Db Verbindungen schlieen
  rex_sql::disconnect(null);

  echo $content;
}

/**
 * Prft, ob sich dateien gendert haben
 *
 * XHTML 1.1: HTTP_IF_MODIFIED_SINCE feature
 *
 * @param $REX_ARTICLE rex_article Den zu sendenen Artikel
 */
function rex_send_last_modified($REX_ARTICLE)
{
  if($REX_ARTICLE)
    $lastModified = $REX_ARTICLE->getValue('updatedate');
  else
    $lastModified = time();

  $lastModified = date('r', $lastModified);

  // Last-Modified Timestamp gefunden
  // => den Browser anweisen, den Cache zu verwenden
  if(isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) && $_SERVER['HTTP_IF_MODIFIED_SINCE'] == $lastModified)
  {
    while(@ob_end_clean());

    header('HTTP/1.1 304 Not Modified');
    die();
  }

  // Sende Last-Modification time
  header('Last-Modified: ' . $lastModified);
}

/**
 * Prft ob sich der Inhalt einer Seite gendert hat
 *
 * XHTML 1.1: HTTP_IF_NONE_MATCH feature
 *
 * @param $content string Inhalt des Artikels
 */
function rex_send_etag($content)
{
  $cacheKey = md5($content);

  // CacheKey gefunden
  // => den Browser anweisen, den Cache zu verwenden
  if(isset($_SERVER['HTTP_IF_NONE_MATCH']) && $_SERVER['HTTP_IF_NONE_MATCH'] == $cacheKey)
  {
    while(@ob_end_clean());

    header('HTTP/1.1 304 Not Modified');
    exit();
  }

  // Sende CacheKey als ETag
  header('ETag: "' . $cacheKey .'"');
}

/**
 * Kodiert den Inhalt des Artikels in GZIP/X-GZIP
 *
 * XHTML 1.1: HTTP_ACCEPT_ENCODING feature
 *
 * @param $content string Inhalt des Artikels
 */
function rex_send_gzip($content)
{
  // Check if it supports gzip
  if (isset($_SERVER['HTTP_ACCEPT_ENCODING']))
    $encodings = explode(',', strtolower(preg_replace('/\s+/', '', $_SERVER['HTTP_ACCEPT_ENCODING'])));

  if ((in_array('gzip', $encodings) || in_array('x-gzip', $encodings) || isset($_SERVER['---------------'])) && function_exists('ob_gzhandler') && !ini_get('zlib.output_compression'))
  {
    $enc = in_array('x-gzip', $encodings) ? 'x-gzip' : 'gzip';
    $supportsGzip = true;
  }

  if($supportsGzip)
  {
    header('Content-Encoding: '. $enc);
    $content = gzencode($content, 9, FORCE_GZIP);
  }

  return $content;
}

?>